from django.shortcuts import render, redirect, HttpResponseRedirect, get_object_or_404
from django.contrib import messages
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.decorators import login_required
from .forms import AccountForm, UserRegistrationForm, ClubForm, ElevateUserForm, ScreenForm, ShowingForm, FilmForm, CinemaForm, ClubRepresentativeForm, EndOfMonthStatementForm
from .models import Account, User, Club, Screen, Cinema, Film, Showing, ClubRepresentative, EndOfMonthStatement
from django.http import JsonResponse
from django.contrib.auth.models import AbstractUser, Group, Permission
import random
import datetime
    


def get_user_permissions(request):
    """
    Returns:
    perms (str): The user's permission level, as a string.
    '3' for account and cinema manager
    '1' for account manager
    '2' for cinema manager
    '0' for no special permissions
    """
    if request.user.groups.filter(name='account_manager').exists() and request.user.groups.filter(name='cinema_manager').exists():
        perms = '3'
    elif request.user.groups.filter(name='account_manager').exists():
        perms = '1'
    elif request.user.groups.filter(name='cinema_manager').exists():
        perms = '2'
    else:
        return None
    return perms

 
# Create your views here.

def index_view(request):
    perms = get_user_permissions(request)
    print(perms)
    return render(request, 'index.html', {'user': request.user, 'perms': perms})

def no_access_redirect(request):
    perms = get_user_permissions(request)
    return render(request, 'no_access.html')


def login_view(request):
    """
    Handle a user logging in to uweflix.
    
    If the request method is "POST", the function checks the provided username and password against the 
    database to authenticate the user. If the user is authenticated, the user is logged in and redirected 
    to the index page. If the user is not authenticated, an error message is displayed and the login form 
    is shown again. If the request method is not "POST", the login form is displayed.
    
    Returns:
        If the request method is "POST" and the user is not authenticated:
            The login form, with the error message passed in as context.
        If the request method is not "POST":
            The login form.
        Otherwise:
            A redirect to the index page.
    """
    
    # If the user has submitted the form
    if request.method == "POST":
        # Validate the form data
        username = request.POST.get('inputUsername')
        password = request.POST.get('inputPassword')
        user = authenticate(request, username=username, password=password)
        
        # If the user is authenticated
        if user is not None:
            login(request, user)
            return redirect('index')
        
        # If the user is not authenticated
        else:
            error_message = "Invalid username or password"
            context = {'error_message': error_message}
            print(error_message)
            return render(request, 'login.html', context)
    
    # If the user has not submitted the form
    else:
        # Display the login form
        return render(request, 'login.html')

def club_rep_login(request):
    if request.method == 'POST':
        rep_id = request.POST.get('inputRepID')
        print(rep_id)
        password = request.POST.get('inputPassword')
        club_rep = authenticate(request, username=rep_id, password=password)
        print(club_rep)
        if club_rep is not None:
            login(request, club_rep)
            return redirect('index')
        else:
            error_message="Invalid representative ID or password"
            print(error_message)
            context = {'error_message': error_message}
            return render(request, 'club_rep_login.html', context)
        
    else:
        club_reps = ClubRepresentative.objects.all()
        print(club_reps)
        context = {'club_reps': club_reps}
        return render(request, 'club_rep_login.html', context)
        

def logout_view(request):
    logout(request)
    return render(request, 'login.html')

def register_view(request):
    """
    Handle a user registering for uweflix.
    
    If the request method is "POST", the function prompts the user to enter user information
    and password verification before being redirected to the index page if succesful.
    If the request method is not "POST", the login form is displayed.
    
    Returns:
        If the request method is "POST" and the form is not valid:
            The registration form, with the error message passed in as context.
        If the request method is not "POST":
            The registration form form.
        Otherwise:
            A redirect to the index page.
    """
    perms = get_user_permissions(request)
    if request.method == "POST":
        form = UserRegistrationForm(request.POST)
        if form.is_valid():
            user = form.save(commit=True)
            email = form.cleaned_data['email']
            password = form.cleaned_data['password1']
            first_name = form.cleaned_data['first_name']
            last_name = form.cleaned_data['last_name']
            login(request, user)
            messages.success(request, "Registration Successful")
            return redirect('/index/')
        else:
            print(form.errors)
    form = UserRegistrationForm()
    return render(request, 'register.html', {'form': form, 'perms': perms})
        


def create_cinema(request):
    """
    Handle a user creating a cinema.
    
    If the request method is "POST", the function prompts the user to enter cinema information
    before being redirected to the cinema management page if succesful.
    If the request method is not "POST", the cinema form is displayed.
    
    Returns:
        If the request method is "POST" and the form is not valid:
            The cinema form, with the error message passed in as context.
        If the request method is not "POST":
            The cinema form form.
        Otherwise:
            A redirect to the cinema management page.
    """
    perms = get_user_permissions(request)
    if request.method == "POST":
        form = CinemaForm(request.POST)
        if form.is_valid():
            form.save(commit=True)
            return redirect('manage_screens')
        else:
            form = CinemaForm()
            context = {'form': form, 'user': request.user, 'perms': perms}
            return render(request, 'create_cinema.html', context)

    form = CinemaForm()        
    context = {'form': form, 'user': request.user, 'perms': perms}
    return render(request, 'create_cinema.html', context)


def elevate_user_view(request):
    """
    Handle a request to grant permissions.
    
    If the request method is "POST", the function prompts the user to enter the user
    and auth group before being redirected to the index page if succesful.
    If the request method is not "POST", the permissions is displayed.
    
    Returns:
        If the request method is "POST" and the form is not valid:
            The permissions form, with the error message passed in as context.
        If the request method is not "POST":
            The permissions form form.
        Otherwise:
            A redirect to the index page.
    """
    perms = get_user_permissions(request)
    if request.method == 'POST':
        form = ElevateUserForm(request.POST)
        if form.is_valid():
            # Form is valid, process the data and return a response
            username = form.cleaned_data['user']
            user = User.objects.get(username=username)
            group_name = form.cleaned_data['group']

            group, created = Group.objects.get_or_create(name=group_name)
            user.groups.add(group)

            # Save changes to the user instance
            user.save()

            return redirect('index')
        else:
            print('invalid form')
    else:
        form = ElevateUserForm()

    context = {'form': form, 'user': request.user, 'users': User.objects.filter(is_rep=False), 'perms': perms}    
    return render(request, 'elevate_user.html', context)


def view_my_details(request):
    """
    Handle a request for the user to see their details
    
    Returns:
        the users information page
    """
    perms = get_user_permissions(request)
    if request.method == 'GET':
        # Get the current logged-in user
        user = request.user
        # Render the template with the user's details
        return render(request, 'view_details.html', {'user': user, 'perms': perms})


#################### SCREEN MANAGEMENT ########################


def screen_list(request):
    """
    Handle a screen management page request.
    
    This gets all screen and accompanying model data from the database to pass to the html template.
    
    Returns:
        screen management page with accompanying data
    """
    perms = get_user_permissions(request)
    print(perms)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    screens = Screen.objects.all()
    showings = Showing.objects.all()
    context = {'user': request.user, 'screens': screens, 'showings': showings, 'perms': perms}
    return render(request, 'manage_screens.html', context)

def screen_create(request):
    """
    Handle a request to create a new screen.
    
    If the request method is "POST", the function prompts the user to enter screen information 
    before being redirected to the screen management page if succesful.
    If the request method is not "POST", the screen form is displayed.
    
    Returns:
        If the request method is "POST" and the form is not valid:
            The screen form, with the error message passed in as context.
        If the request method is not "POST":
            The screen form form.
        Otherwise:
            A redirect to the screen management page.
    """
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    cinemas = Cinema.objects.all()
    if request.method == 'POST':
        form = ScreenForm(request.POST)
        if form.is_valid():
            form.save()
            form = ScreenForm()
            context = {'form': form, 'user': request.user, 'cinemas': Cinema.objects.all(), 'perms': perms}
            return redirect('manage_screens')
    form = ScreenForm()
    context = {'form': form, 'user': request.user, 'cinemas': Cinema.objects.all(), 'perms': perms}
    return render(request, 'create_screen.html', {'form': form, 'cinemas': cinemas})

def screen_update(request, pk):
    """
    Handle a user requesting to update a screen
    
    Returns:
        If the form is invalid:
            screen form
        otherwise:
            redirect to screen management
    """
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    screen = get_object_or_404(Screen, pk=pk)
    if request.method == 'POST':
        form = ScreenForm(request.POST, instance=screen)
        if form.is_valid():
            form.save()
            return redirect('screen_list')
        
    form = ScreenForm(instance=screen)
    context = {'form': form, 'user': request.user, 'screen': screen, 'perms': perms}    
    return render(request, 'update_screen.html', context)

def screen_delete(request, pk):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    screen = get_object_or_404(Screen, pk=pk)
    screen.delete()
    return redirect('screen_list')

def showing_create(request):
    """
    Handle a user creating a showing from an accompanying screen
    
    Returns:
        If the request method is "POST" and the form is not valid:
            The showing form, with the error message passed in as context.
        If the request method is not "POST":
            The showing form form.
        Otherwise:
            A redirect to the screen management page.
    """
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    films = Film.objects.all()
    screens = Screen.objects.all()
    if request.method == 'POST':
        form = ShowingForm(request.POST)
        print(form.errors)
        if form.is_valid():
            form.save(commit=True)
            return redirect('manage_screens')
        else:
            print('invalid form')
    form = ShowingForm()
    context = {'form': form, 'user': request.user, 'films': films, 'screens': screens, 'perms': perms}
    return render(request, 'create_showing.html', context)

def showing_update(request, pk):
    """
    Handle a user requesting to update a showing
    
    Returns:
        If the form is invalid:
            showing form
        otherwise:
            redirect to screen management
    """
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    showing = get_object_or_404(Showing, pk=pk)
    films = Film.objects.all()
    screens = Screen.objects.all()
    if request.method == 'POST':
        form = ShowingForm(request.POST, instance=showing)
        if form.is_valid():
            form.save()
            return redirect('manage_screens')
    form = ShowingForm(instance=showing)
    context = {'form': form, 'user': request.user, 'films': films, 'screens': screens, 'perms': perms}
    return render(request, 'update_showing.html', context)

def showing_delete(request, pk):
    perms = get_object_or_404(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    showing = get_object_or_404(Showing, pk=pk)
    showing.delete()
    return redirect('manage_screens')

############### ACCOUNT MANAGEMENT #####################

def clubs_list(request):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    clubs = Club.objects.all()
    accounts = Account.objects.all()
    #representatives = User.objects.all().filter(club_rep=True)
    context = {'user': request.user, 'clubs': clubs, 'accounts': accounts, 'perms': perms}
    return render(request, 'manage_clubs.html', context)


def club_update(request, pk):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    club = get_object_or_404(Club, pk=pk)

    # Check if the request method is POST
    if request.method == 'POST':
        # Bind the form with the POST data
        form = ClubForm(request.POST, instance=club)
        # Check if the form is valid
        if form.is_valid():
            # Save the form and redirect to the club list page
            form.save()
            return redirect('manage_clubs')
    # If the request is not POST, or the form is invalid,
    # render the update page with the club instance and the form
    form = ClubForm(instance=club)
    context = {'form': form, 'club': club, 'pk': pk, 'perms': perms}
    return render(request, 'update_club.html', context)


def club_delete(request, pk):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    club = get_object_or_404(Club, pk=pk)
    # Get the representative and account associated with the club
    representative = club.representative
    account = club.account
    # Delete the club
    club.delete()
    # Delete the representative and account
    representative.delete()
    account.delete()
    return redirect('manage_clubs')


def club_create(request):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    if request.method == 'POST':
        form = ClubForm(request.POST)
        if form.is_valid():
            club = form.save(commit=True)
            representative_id = club.name[0] + str(random.randint(1,1000))
            club_rep = ClubRepresentative.objects.create(username=club.name, representative_id=representative_id, password='default')
            club_rep.is_rep=True
            club_rep.save()
            club_account = Account.objects.create(payment_card_number='', payment_card_expiry_date=datetime.date(1984, 1, 1), discount_rate=0)
            
            club.representative = club_rep
            club.account = club_account
            club.save()
            return redirect('/index/')
    form = ClubForm()
    context = {'form': form, 'user': request.user, 'perms': perms}
    return render(request, 'create_club.html', context)


def account_create(request):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    clubs = Club.objects.all()
    ### USER SUBMITS FORM ###
    if not request.user.groups.all().filter(name='account_manager').exists():
        print(request.user.user_permissions)
        return redirect('no_access')

    if request.method == 'POST':
        # code to handle form submission and create a new account
        form = AccountForm(request.POST)
        print(form.errors)
        if form.is_valid():
            account = form.save(commit=True)
            club = form.cleaned_data['club']
            club.account = account
            club.save()
            return redirect('manage_clubs')
        else:
            # render the form with errors
            context = {'form': form, 'user': request.user, 'clubs': clubs, 'perms': perms}
            return render(request, 'create_account.html', context)
    else:
        ### USER REQUESTS FORM ###
        # render the empty form
        form = AccountForm()
        context = {'form': form, 'user': request.user, 'clubs': clubs, 'perms': perms}
        return render(request, 'create_account.html', context)


def account_update(request, pk):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    account = get_object_or_404(Account, pk=pk)
    if request.method == 'POST':
        form = AccountForm(request.POST, instance=account)
        # Check if the form is valid
        if form.is_valid():
            # Save the form and redirect to the club list page
            form.save()
            return redirect('manage_clubs')
    # If the request is not POST, or the form is invalid,
    # render the update page with the club instance and the form
    form = AccountForm(instance=account)
    context = {'form': form, 'user': request.user, 'account': account, 'pk': pk, 'perms': perms}
    return render(request, 'update_account.html', context)


def account_delete(request, pk):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    account = get_object_or_404(Account, pk=pk)
    if request.method == 'POST':
        account.delete()
    return redirect('manage_clubs')
        
def representative_update(request, pk):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    club = get_object_or_404(Club, pk=pk)
    representative = club.representative.clubrepresentative
    if request.method == 'POST':
        print('request post')
        form = ClubRepresentativeForm(request.POST, instance=representative)
        if form.is_valid():
            print('club rep form valid')
            form.save()
            return redirect('manage_clubs')
        else:
            print(form.errors)
    else:
        form = ClubRepresentativeForm(instance=representative, initial={'representative_id': representative.representative_id})
    context = {'form': form, 'user': request.user, 'club': club, 'representative': representative, 'perms': perms}
    return render(request, 'update_representative.html', context)



def statements_list(request, pk):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    account = Account.objects.get(pk=pk)
    statements = EndOfMonthStatement.objects.filter(account__pk=pk)
    print(f'statements: {statements}')
    context = {'user': request.user, 'statements': statements, 'perms': perms}
    return render(request, 'manage_statements.html', context)


def statement_generate(request, pk):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    account = Account.objects.get(pk=pk)
    statement = Account.create_statement(account_id=pk)
    statement.save()
    
    statements = EndOfMonthStatement.objects.filter(account__pk=pk)
    context = {'user': request.user, 'statements': statements, 'perms': perms}
    return render(request, 'manage_statements.html', context)

def statement_update(request, pk, st_pk):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    account = Account.objects.get(pk=pk)
    statement = EndOfMonthStatement.objects.get(pk=st_pk)
    statements = EndOfMonthStatement.objects.filter(account__pk=pk)
    
    if request.method == 'POST':
        form = EndOfMonthStatementForm(request.POST, instance=statement)
        # Check if the form is valid
        if form.is_valid():
            # Save the form and redirect to the club list page
            amended_statement = form.save()
            new_outstanding = EndOfMonthStatement.update_outstanding(amended_statement.id)
            amended_statement.outstanding = new_outstanding
            amended_statement.save()
            
            context = {'user': request.user, 'statements': statements, 'perms': perms}
            return render(request, 'manage_statements.html', context)
    # If the request is not POST, or the form is invalid,
    # render the update page with the club instance and the form
    form = EndOfMonthStatementForm(instance=statement)
    context = {'form': form, 'user': request.user, 'account': account, 'pk': pk, 'st_pk': st_pk, 'statement': statement, 'perms': perms}
    return render(request, 'update_statement.html', context)


def statement_delete(request, pk, st_pk):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    account = Account.objects.get(pk=pk)
    statement = EndOfMonthStatement.objects.get(pk=st_pk)
    statement.delete()
    statements = EndOfMonthStatement.objects.filter(account__pk=pk)
    context = {'user': request.user, 'statements': statements, 'perms': perms}
    return render(request, 'manage_statements.html', context)

### FILM MANAGEMENT


def film_create(request, pk=None):
    perms = get_user_permissions(request)
    if perms == '0' or perms == '1':
        return redirect('no_access')
    
    success = False
    if request.method == 'POST':
        form = FilmForm(request.POST)
        if form.is_valid():
            form.save()
            form = FilmForm()

    form = FilmForm()
    context = {'form': form, 'user': request.user, 'pk': pk, 'perms': perms}
    return render(request, 'create_film.html', context)
    


